@@ -9,7 +9,7 @@ For running the demo you need PyGI (aka python-gi).
 You will need to use Python 2.x because
 cairo.ImageSurface.create_for_data is not yet available in Python 3,
 see
-<http://cairographics.org/documentation/pycairo/3/reference/surfaces.html#cairo.ImageSurface.create_for_data>.
+<https://cairographics.org/documentation/pycairo/3/reference/surfaces.html#cairo.ImageSurface.create_for_data>.
 
 and this was not tested in Python 3.
 
@@ -18,13 +18,13 @@ device is required::
 
     ./dxmain.py -dBATCH -sDEVICE=display -dFitPage /path/to/eps_or_pdf
 
-more options: http://www.ghostscript.com/doc/current/Use.htm#Output_device
+more options: https://ghostscript.readthedocs.io/en/gs10.0.0/Use.html#Output_device
 
 see the main function to change the setup of the display device
 
 """
 # 
-# This is a python port from dxmain.c by artifex http://www.artifex.com
+# This is a python port from dxmain.c by artifex https://www.artifex.com
 # dxmain.c is int the ghostscript repositoty at ghostpdl/gs/psi/dxmain.c
 #
 # Please do not take this as an example for a GTK programm written in
@@ -44,7 +44,7 @@ see the main function to change the setup of the display device
 # General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program. If not, see <http://www.gnu.org/licenses/>.
+# along with this program. If not, see <https://www.gnu.org/licenses/>.
 #
 
 __author__ = "Lasse Fister <commander@graphicore.de>"
@@ -55,6 +55,8 @@ __licence__ = "GNU General Public License version 3 (GPL v3)"
 import ghostscript._gsprint as gs
 import ctypes as c
 
+import gi
+gi.require_version('Gtk', '3.0')
 from gi.repository import Gtk, Gdk, GLib, GdkPixbuf
 
 # fails when being used:
@@ -64,10 +66,19 @@ import cairo
 
 import sys
 from array import array
+import locale
+
+if sys.version_info >= (3,):
+    raise SystemExit(
+        "Sorry! This example still requires Python 2 since some underlying cairo "
+        "function is still not available in Python 3. If you know how to "
+        "solve this, please send in a patch or merge-request. Thanks.")
 
 
+encoding = locale.getpreferredencoding()
 
-start_string = "systemdict /start get exec\n"
+
+start_string = b"systemdict /start get exec\n"
 
 #####################################################################
 # stdio functions
@@ -121,7 +132,7 @@ def read_stdin_handler(channel, condition, inputBuffer):
 def _gsdll_stdin(instance, buf, length):
     inputBuffer = Stdin_buf(buf, length, -1) # buf, len, count
     
-    channel = sys.stdin
+    channel = sys.stdin.buffer  # bytes buffer
     # (fd, condition, callback, user_data=None) -> source id
     # callable receives (fd, condition, user_data)
     # Arranges for the fd to be monitored by the main loop for the
@@ -198,9 +209,6 @@ def window_draw(widget, cr, img):
     this callback is called via: img.darea.connect('draw', window_draw, img)
     """
     if img and img.window and img.buf:
-        bgcol = widget.get_style_context().get_background_color(Gtk.StateFlags.NORMAL)
-        cr.set_source_rgba(bgcol.red, bgcol.blue, bgcol.green, bgcol.alpha)
-        cr.paint()
         if img.rgbbuf:
             cairo_surface = cairo.ImageSurface.create_for_data(img.rgbbuf, cairo.FORMAT_RGB24, img.width, img.height, img.width * 4)
             cr.set_source_surface(cairo_surface, 0, 0)
@@ -217,7 +225,7 @@ def widget_delete(widget, *args):
 
 def window_create(img):
     """ Create a gtk window """
-    img.window = Gtk.Window(Gtk.WindowType.TOPLEVEL)
+    img.window = Gtk.Window(type=Gtk.WindowType.TOPLEVEL)
     img.window.set_title("python gs");
     
     img.vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
@@ -229,11 +237,11 @@ def window_create(img):
     img.darea = Gtk.DrawingArea()
     img.darea.show()
     
-    img.scroll = Gtk.ScrolledWindow(None, None)
+    img.scroll = Gtk.ScrolledWindow()
     img.scroll.show()
     
     img.scroll.set_policy(Gtk.PolicyType.ALWAYS, Gtk.PolicyType.ALWAYS)
-    img.scroll.add_with_viewport(img.darea)
+    img.scroll.add(img.darea)
     img.vbox.pack_start(img.scroll, True, True, 0)
     
     img.darea.connect('draw', window_draw, img)
@@ -251,9 +259,15 @@ def window_resize(img):
         # desktop toolbars, and if possible a little larger than
         # the image to allow room for the scroll bars.
         # We don't know the width of the scroll bars, so just guess.
+        display = Gdk.Display.get_default()
+        monitor = display.get_primary_monitor()
+        geometry = monitor.get_geometry()
+        scale_factor = monitor.get_scale_factor()
+        width = scale_factor * geometry.width
+        height = scale_factor * geometry.height
         img.window.set_default_size(
-            min(Gdk.Screen.width()-96, img.width+24),
-            min(Gdk.Screen.height()-96, img.height+24)
+            min(width-96, img.width+24),
+            min(height-96, img.height+24)
         )
 
 def window_separation(img, sep):
@@ -899,35 +913,39 @@ def main(argv):
     
     nargv = [argv[0], dformat] + argv[1:]
     
+    # For the low-level interface arguments have to be bytes. Encode
+    # them using local encoding to save calling set_arg_encoding().
+    nargv = [a.encode(encoding) for a in nargv]
+
     #run Ghostscript 
+    instance = gs.new_instance()
+    gs.set_stdio(instance, gsdll_stdin, None, None)
+    if use_gui:
+        gs.set_display_callback(instance, c.byref(display))
+
+    code = 0
     try:
-        instance = gs.new_instance()
-        gs.set_stdio(instance, gsdll_stdin, None, None)
-        if use_gui:
-            gs.set_display_callback(instance, c.byref(display))
-        code = gs.init_with_args(instance, nargv)
-        if code == 0:
-            code = gs.run_string(instance, start_string)
-        code1 = gs.exit(instance)
-        if code == 0 or code == gs.e_Quit:
-            code = code1
-        if code == gs.e_Quit:
-            code = 0 # user executed 'quit'
-        gs.delete_instance(instance)
+        gs.init_with_args(instance, nargv)
+        gs.run_string(instance, start_string)
     except gs.GhostscriptError as e:
+        print("ERROR:", e, file=sys.stderr)
         code = e.code
-        sys.stderr.write(e.message)
     finally:
-        exit_status = 0;
-        if code in [0, gs.e_Info, gs.e_Quit]:
-            pass
-        elif code == gs.e_Fatal:
-            exit_status = 1
-        else:
-            exit_status = 255
+        code1 = gs.exit(instance)
+        if code in (0,  gs.e_Quit):
+            code = code1
+
+    gs.delete_instance(instance)
+
+    if code in [0, gs.e_Info, gs.e_Quit]:
+        exit_status = 0
+    elif code == gs.e_Fatal:
+        exit_status = 1
+    else:
+        exit_status = 255
     return exit_status
 
 if __name__ == '__main__':
     code = main(sys.argv)
-    sys.stdout.write('\n') # or bash will get out of sync ...
+    print(file=sys.stdout) # or bash will get out of sync ...
     sys.exit(code)
