@@ -3,9 +3,9 @@
 if [ -z "$PYTHON" ] ; then PYTHON=python ; fi
 
 cd $(dirname $0)
-PYTHONPATH=$(realpath ..):$PYTHONPATH
+export PYTHONPATH=$(realpath ..):$PYTHONPATH
 
 $PYTHON ./gs.py \
     -dNOPAUSE -dBATCH -dSAFER -sDEVICE=pdfwrite -sOutputFile=/tmp/out.pdf \
-    -c .setpdfwrite -f ./test.ps
+    -f ./test.ps
 
