@@ -3,6 +3,6 @@
 if [ -z "$PYTHON" ] ; then PYTHON=python ; fi
 
 cd $(dirname $0)
-PYTHONPATH=$(realpath ..):$PYTHONPATH
+export PYTHONPATH=$(realpath ..):$PYTHONPATH
 
 $PYTHON ./lowlevel1.py
