@@ -3,8 +3,8 @@
 if [ -z "$PYTHON" ] ; then PYTHON=python ; fi
 
 cd $(dirname $0)
-PYTHONPATH=$(realpath ..):$PYTHONPATH
+export PYTHONPATH=$(realpath ..):$PYTHONPATH
 
-$PYTHON ./lowlevel2.py \
+$PYTHON ./lowlevel2.py -q \
     -dNOPAUSE -dBATCH -dSAFER -sDEVICE=pdfwrite -sOutputFile=/tmp/out.pdf \
-    -c .setpdfwrite -f ./test.ps
+    -f ./test.ps
